﻿using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing.Design;
using Wisej.Web;

namespace Wisej.DataBoundCheckedListBox
{
	public class DataBoundCheckedListBox : CheckedListBox
	{
		/// <summary>
		/// Returns or sets the property to bind to the check state for the items in the <see cref="Wisej.Web.ListControl" />.
		/// </summary>
		/// <returns>A <see cref="System.String" /> specifying the name of an object property that is contained in the collection specified by the <see cref="Wisej.Web.ListControl.DataSource" /> property. The default is an empty string (""). </returns>
		[DefaultValue("")]
		[Category("CatData")]
		[Description("ListControlIconMemberDescr")]
		[TypeConverter("System.Windows.Forms.Design.DataMemberFieldConverter, System.Design, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
		[Editor("System.Windows.Forms.Design.DataMemberFieldEditor, System.Design, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(UITypeEditor))]
		public virtual string CheckStateMember
		{
			get
			{
				return this._checkStateMember.BindingMember;
			}
			set
			{
				if (value == null)
					value = "";

				if (this.CheckStateMember != value)
					this._checkStateMember = new BindingMemberInfo(value);
			}
		}
		private BindingMemberInfo _checkStateMember;

		protected override void SetItemsCore(IList value)
		{
			base.SetItemsCore(value);
			UpdateCheckState();
		}

		protected override void SetItemCore(int index, object value)
		{
			base.SetItemCore(index, value);
			UpdateCheckState(index);
		}

		protected override void RefreshItems()
		{
			base.RefreshItems();
			UpdateCheckState();
		}

		protected override void RefreshItem(int index)
		{
			base.RefreshItem(index);
			UpdateCheckState(index);
		}

		private bool _inUpdatingData;

		protected override void OnAfterItemCheck(ItemCheckEventArgs e)
		{
			base.OnAfterItemCheck(e);

			if (!this._inUpdatingData)
			{
				var dataManager = base.DataManager;
				if (dataManager != null)
				{
					dataManager.GetItemProperties()[this.CheckStateMember]
						.SetValue(dataManager.List[e.Index], Convert.ToBoolean(e.NewValue));
				}
			}
		}

		private void UpdateCheckState()
		{
			var dataManager = base.DataManager;
			if (dataManager != null)
			{
				this._inUpdatingData = true;
				for (int i = 0, l = dataManager.Count; i < l; i++)
				{
					SetItemChecked(i, Convert.ToBoolean(FilterItemOnProperty(dataManager.List[i], this.CheckStateMember)));
				}
				this._inUpdatingData = false;
			}
		}

		private void UpdateCheckState(int index)
		{
			this._inUpdatingData = true;
			var dataManager = base.DataManager;
			SetItemChecked(index, Convert.ToBoolean(FilterItemOnProperty(dataManager.List[index], this.CheckStateMember)));
			this._inUpdatingData = false;
		}
	}
}